#!/bin/bash

#$1: decide-vm name
#$2: decide-vm external_ip_address
#$3: decide-vm internal_ip_address
#$4: wan consul-join-ip
#$5: adapt host public ip and port
#Optional arguments:
#$6: Private Docker registry ip
#$7: Private Docker registry port

##WAITING FOR THE SYSTEM TO BE READY
#sleep 20

MAX_RETRIES=120
#SECS between retries
SLEEP_INTERVAL=5
#MAX retries before removing lock abruptly
MAX_SOFT_RETRIES=12
RETRIES=0
RESULT=0

echo "About to curl for declaring that VM is up and running, and in a 'configuring' state"
curl --max-time 4 -X POST $5/vm/vmlist/$2/configuring
echo "About to curl for step 1: cleanup"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'", "status": "configuring", "step": 1, "steps": 9, "desc": "clean before update"}' $5/vm/$2/status || RESULT=$?


if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi

sudo apt-get clean
while [ $? != 0 ] && [ $RETRIES -lt $MAX_RETRIES ]; do 
    echo "apt-get clean failed with code $?"  >> /tmp/scripts/install.log
    sleep $SLEEP_INTERVAL
    if [ $RETRIES -gt $MAX_SOFT_RETRIES ]; then
        echo "*** Waited too much: removing lock abruptly! ***"
        sudo rm -f /var/lib/dpkg/lock
    fi
    RETRIES=$((RETRIES + 1))
    echo "Retry n. $RETRIES "  >> /tmp/scripts/install.log
    sudo apt-get clean
done

if [ $RETRIES -eq $MAX_RETRIES ]; then
    echo "Error in apt-get clean operation: giving up!"
    exit 1
fi

echo "About to curl for step 2: update"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "configuring", "step": 2, "steps": 9, "desc": "update packages"}' $5/vm/$2/status || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi


RETRIES=0
sudo apt-get install -f
while [ $? != 0 ] && [ $RETRIES -lt $MAX_RETRIES ]; do 
    echo "apt-get install failed with code $?"  >> /tmp/scripts/install.log
    sleep $SLEEP_INTERVAL
    if [ $RETRIES -gt $MAX_SOFT_RETRIES ]; then
        echo "*** Waited too much: removing lock abruptly! ***"
        sudo rm -f /var/lib/dpkg/lock
    fi
    sudo rm -rf /var/lib/apt/lists/* -vf
    RETRIES=$((RETRIES + 1))
    echo "Retry n. $RETRIES "  >> /tmp/scripts/install.log
    sudo apt-get install -f
done

if [ $RETRIES -eq $MAX_RETRIES ]; then
    echo "Error in apt-get install operation: giving up!"
    exit 1
fi

RETRIES=0
sudo apt-get update -y 
while [ $? != 0 ] && [ $RETRIES -lt $MAX_RETRIES ]; do 
    echo "apt-get update failed with code $?"  >> /tmp/scripts/install.log
    sleep $SLEEP_INTERVAL
    if [ $RETRIES -gt $MAX_SOFT_RETRIES ]; then
        echo "*** Waited too much: removing lock abruptly! ***"
        sudo rm -f /var/lib/dpkg/lock
    fi
    RETRIES=$((RETRIES + 1))
    echo "Retry n. $RETRIES "  >> /tmp/scripts/install.log
    sudo apt-get update -y 
done

if [ $RETRIES -eq $MAX_RETRIES ]; then
    echo "Error in apt-get update operation: giving up!"
    exit 1
fi


##START INSTALLATION AND CONFIGURATION PROCESS
chmod +x /tmp/scripts/install-docker-and-compose.sh
chmod +x /tmp/scripts/configure-sec.sh
chmod +x /tmp/scripts/install-and-run-consul.sh
chmod +x /tmp/scripts/registry-login-and-pull.sh

echo "Configuring security..." >> /tmp/scripts/install.log

echo "About to curl for step 3: security"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "configuring", "step": 3, "steps": 9, "desc": "configuring security"}' $5/vm/$2/status || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi

/tmp/scripts/configure-sec.sh $2 || RESULT=$?
if [ $RESULT -ne 0 ] ; then
	echo "Error in configuring security"  >> /tmp/scripts/install.log
else
	echo "...security configuration done"  >> /tmp/scripts/install.log
fi

echo "installing docker and docker-compose..."  >> /tmp/scripts/install.log

echo "About to curl for step 4: install docker and docker compose"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "configuring", "step": 4, "steps": 9, "desc": "installing docker and docker compose"}' $5/vm/$2/status || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi

/tmp/scripts/install-docker-and-compose.sh || RESULT=$?
if [ $RESULT -ne 0 ] ; then
	echo "Error in installing docker and docker-compose"  >> /tmp/scripts/install.log
else
	echo "...docker and docker-compose installation done"  >> /tmp/scripts/install.log
fi

echo "About to curl for step 5: configure docker as service"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "configuring", "step": 5, "steps": 9, "desc": "configuring docker as service"}' $5/vm/$2/status || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi

echo "Configuring docker as service..."  >> /tmp/scripts/install.log
sudo cp /tmp/scripts/docker.service /lib/systemd/system/docker.service
sudo systemctl daemon-reload
sudo systemctl restart docker || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Error in configuring docker as service"  >> /tmp/scripts/install.log
else
	echo "...docker configuration as a service done"  >> /tmp/scripts/install.log
fi

echo "Installing and running consul..." >> /tmp/scripts/install.log

echo "About to curl for step 6: install consul"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "configuring", "step": 6, "steps": 9, "desc": "installing consul"}' $5/vm/$2/status || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi

/tmp/scripts/install-and-run-consul.sh $1 $2 $3  || RESULT=$?
if [ $RESULT -ne 0 ] ; then
	echo "Error in installing consul"  >> /tmp/scripts/install.log
else
	echo "..consul installed"  >> /tmp/scripts/install.log
fi

echo "About to curl for step 7: joining cluster"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "configuring", "step": 7, "steps": 9, "desc": "joining consul wan cluster"}' $5/vm/$2/status || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi

echo "Joining the wan cluster at $4..." >> /tmp/scripts/install.log
consul join -http-addr=$3:8500 -wan $4 || RESULT=$?
if [ $RESULT -ne 0 ] ; then
	echo "Error in joining consul wan cluster"  >> /tmp/scripts/install.log
else
	echo "..joined consul wan cluster"  >> /tmp/scripts/install.log
fi

if [ $# -ge 7 ]
  then
	echo "About to curl for step 8: configuring access to private registry"
	curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "configuring", "step": 8, "steps": 9, "desc": "configuring access to private registry"}' $5/vm/$2/status || RESULT=$?

	if [ $RESULT -ne 0 ] ; then
		echo "Could not log step status"
	fi

    /tmp/scripts/registry/configure-registry-cert.sh $6 $7 || RESULT=$?
	if [ $RESULT -ne 0 ] ; then
		echo "Error in configuring access to private registry"  >> /tmp/scripts/install.log
	else
		echo "..configured access to private registry"  >> /tmp/scripts/install.log
	fi
fi

echo "About to curl for step 9: ready"
curl --max-time 4 -X POST -H 'Content-Type: application/json;charset=UTF-8'  -d '{"vmName": "'$1'", "vmPublicIp": "'$2'","status": "ready", "step": 9, "steps": 9, "desc": "ready for microservices installation"}' $5/vm/$2/status || RESULT=$?

if [ $RESULT -ne 0 ] ; then
	echo "Could not log step status"
fi
echo "About to curl for declaring that VM is up and running, and in a 'ready' state"
curl --max-time 4 -X POST $5/vm/vmlist/$2/ready

echo "init-vm.sh done"


